#!/bin/bash
# network-setup - simple frontend to various connection tools
# (C) Klaus Knopper Sep 2006

PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin"
export PATH

XDIALOG_HIGH_DIALOG_COMPAT=1
export XDIALOG_HIGH_DIALOG_COMPAT

# Get root
[ "`id -u`" != "0" ] && exec sudo "$0" "$@"

TMP="/tmp/network-setup.$$.tmp"

bailout(){
rm -f "$TMP" "$TMP".done
exit $1
}

DIALOG="dialog"
[ -n "$DISPLAY" ] && [ -x /usr/bin/Xdialog ] && DIALOG="Xdialog --cr-wrap"

trap bailout 1 2 3 15

# LANGUAGE etc.
if [ -r /etc/sysconfig/i18n ]; then
 . /etc/sysconfig/i18n
elif [ -r /etc/default/locale ]; then
 . /etc/default/locale
fi

[ -n "$LANGUAGE" ] || LANGUAGE="$LANG"

# Read last connection, if possible
LASTCONNECT=""
[ -f /etc/sysconfig/network-setup ] && . /etc/sysconfig/network-setup

# progress "Message" seconds_expected
progress(){
if [ -n "$1" ]; then
 case "$LANGUAGE" in
  de*|at*|ch*) T="Bitte warten..." ;;
  *) T="Please wait..." ;;
 esac
 # Show progressbar
 rm -f "$TMP.done"
 expected="$2"
 [ -n "$expected" ] && ratio="$((expected/10))+2" || ratio=4
 status=0
 ( while [ ! -e "$TMP.done" ]; do echo "$(((status=(10000-status)/ratio+status)/100))"; sleep 1; done | $DIALOG --title "$1" --gauge "$T" 8 75 0 ) &
else
 # Kill previous
 touch "$TMP.done" ; wait ; rm -f "$TMP.done"
fi
}

# check [address-to-ping] [numtries] returns
# true if we are already connected to the network, false if not
check(){
TRY="$1"
count="${2:-1}"
 case "$LANGUAGE" in
  de*|at*|ch*) M="Netzwerk-Check..." ;;
  *) M="Network-Check..." ;;
 esac
progress "$M" "$((count * 4 + 2))"
# Check network status
STATUS=""
while [ "$((count--))" -gt "0" ]; do
 GW="$(route -n | awk '/^0\.0\.0\.0/{print $2}')"
 GWDEV="$(route -n | awk '/^0\.0\.0\.0/{print $NF}')"
 if [ -n "$TRY" ]; then
  # If using GPRS, the Gateway is NOT pingable, so please use an external IP.
  NMAP="$(nmap -sP --host_timeout 4000 --max_rtt_timeout 4000 "$TRY" 2>/dev/null)"
  if [ "$?" = 0 ]; then
   echo "$NMAP" | grep -q "down" >/dev/null 2>&1 || { STATUS="online" && break; }
  elif [ -n "$GW" ]; then
   STATUS="maybe online" ; break
  fi
 elif [ -n "$GW" ]; then
  STATUS="maybe online" ; break
 fi
 sleep 2
done
progress ""
[ -n "$STATUS" ] && return 0 || return 1
}

case "$LANGUAGE" in
de*|at*|ch*)
TITLE1="Netzwerk-Setup"
MESSAGE1="Bitte auswhlen:"
MODEMSETUP="/dev/modem-Schnittstelle (neu) konfigurieren"
NETCARD="LAN (Router) Einstellungen"
WLCARD="WLAN (Funknetz) Einstellungen"
GPRS="Einwahl per Handy/GPRS+UMTS"
DSL="Einwahl per DSL"
ISDN="Einwahl per ISDN"
MODEM="Einwahl per Analogmodem"
EXIT="Beenden"
SKIP="berspringen"
MESSAGE_RECONNECT0="Automatisch neu verbinden zum Internet mit \""
MESSAGE_RECONNECT1="\"?\n\n\"$SKIP\" fr manuelles Setup."
LABEL_RECONNECT="Verbindung wiederherstellen"
;;
*)
TITLE1="Network Setup"
MESSAGE1="Please select:"
MODEMSETUP="(re)configure /dev/modem device"
NETCARD="LAN (router) setup"
WLCARD="WLAN (wireless) setup"
GPRS="Dial via cellphone/GPRS+UMTS"
DSL="Dial via DSL-adapter"
ISDN="Dial via ISDN"
MODEM="Dial via analog modem"
EXIT="Quit"
SKIP="Skip"
MESSAGE_RECONNECT0="Automatic reconnect to the Internet using \""
MESSAGE_RECONNECT1="\"?\n\nHit \"$SKIP\" for manual setup."
LABEL_RECONNECT="Reconnect"
;;
esac

# Shortcut description
TYPES=( modemlink "$MODEMSETUP" \
netcardconfig "$NETCARD" \
wlcardconfig "$WLCARD" \
gprsconnect "$GPRS" \
pppoeconf "$DSL" \
isdntool "$ISDN" \
modemtool "$MODEM" )

type modemtool >/dev/null 2>&1 || { unset TYPES[13]; unset TYPES[12]; }
type isdntool >/dev/null 2>&1 || { unset TYPES[11]; unset TYPES[10]; }
type pppoeconf >/dev/null 2>&1 || { unset TYPES[9]; unset TYPES[8]; }
type gprsconnect >/dev/null 2>&1 || { unset TYPES[7]; unset TYPES[6]; }
type wlcardconfig >/dev/null 2>&1 || { unset TYPES[5]; unset TYPES[4]; }
type netcardconfig >/dev/null 2>&1 || { unset TYPES[3]; unset TYPES[2]; }
type modemlink >/dev/null 2>&1 || { unset TYPES[1]; unset TYPES[0]; }

# Call main with an optional IP address to check for reachability
main(){
CHECK=""
STATUS=""
if [ "$1" != "nocheck" ]; then
 [ "$1" = "check" ] && CHECK="true" && shift
 if check "$1" "4"; then
  if [ -n "$CHECK" ]; then # check ONLY
   bailout 0 # exit with success code if we are already connected
  fi
  STATUS="Online ($GWDEV)"
 else
  if [ -n "$CHECK" ]; then # check ONLY
   bailout 1 # exit with error code if we aren't
  fi
  STATUS="Offline"
 fi
fi

# If we are here, there is no network present, and we are not in "check only" mode either.
# Check if user wants to reconnect
checkreconnect && bailout 0

# Language-dependent Messages
case "$LANGUAGE" in
de*|at*|ch*)
STATUS="${STATUS:+Status: $STATUS}"
;;
*)
STATUS="${STATUS:+Current state: $STATUS}"
;;
esac

rm -f "$TMP"
$DIALOG --clear --cancel-label "$EXIT" --title "$TITLE1" --menu "$STATUS
$MESSAGE1" 19 75 10 "${TYPES[@]}" 2>"$TMP" || bailout 1
read TYPE <"$TMP"
TYPE="${TYPE#\"}"; TYPE="${TYPE%\"}"
rm -f "$TMP"

numtries="4"
case "$TYPE" in
 modemlink)    numtries="0"  ;; # No check, continue
 gprsconnect)  killall pump >/dev/null 2>&1 ; numtries="1"  ;; # gprsconnect waits
 isdntool)     killall pump >/dev/null 2>&1 ; numtries="10"  ;;
 modemtool)    killall pump >/dev/null 2>&1 ; numtries="1" ;; # modemtol waits
 pppoeconf)    killall pump >/dev/null 2>&1 ; numtries=""
               # Bugfix: modconf needs terminal
               [ -n "$DISPLAY" ] && TYPE="x-terminal-emulator -e pppoeconf" ;;
esac
 
LASTCONNECT="$TYPE"
echo "LASTCONNECT=\"$LASTCONNECT\"" > /etc/sysconfig/network-setup 2>/dev/null

eval $TYPE || return 1

# Force connect through DNS lookup in case of ISDN dial-on-demand (not a real ping)
case "$TYPE" in isdntool) nmap -sP --host_timeout 4000 --max_rtt_timeout 4000 knopper.net >/dev/null 2>&1 & ;; esac

[ -n "$numtries" ] || bailout 0
[ "$numtries" -gt "0" ] && check "$1" "$numtries" && bailout 0
return 1
}

checkreconnect(){
if [ -n "$LASTCONNECT" ]; then
 count=0
 found=""
 LASTDESCRIPTION=""
 for i in "${TYPES[@]}"; do
  [ -n "$found" ] && LASTDESCRIPTION="$i" && break
  [ "$((count++ % 2))" != "0" ] && continue
  [ -n "$i" ] && case "$LASTCONNECT" in *$i*) found="yes";; esac
 done
 [ -n "$LASTDESCRIPTION" ] || LASTDESCRIPTION="$LASTCONNECT"

 case "$LASTCONNECT" in
  *pppoeconf*) LASTCONNECT="pon dsl-provider";;
            *) LASTCONNECT="$LASTCONNECT reconnect";;
 esac
  
 if $DIALOG --clear --ok-label "$LABEL_RECONNECT" --cancel-label "$SKIP" --title "$TITLE1" --yesno "$MESSAGE_RECONNECT0$LASTDESCRIPTION$MESSAGE_RECONNECT1" 10 75; then
  $LASTCONNECT
  check
  return "$?"
 else
  rm -f /etc/sysconfig/network-setup
 fi
fi
return 1
}

while true; do main "$@"; done
